"""
math_trainer.py
Train your times tables.
Initial Features:
* Print out times table for a given number.
* Limit tables to a lower number (default is 1)
and an upper number (default is 12).
* Pose test questions to the user
* Check whether the user is right or wrong
* Track the users score.
Brendan Scott
February 2015
"""

#### Imports Section
import random
import sys  #H
import time #I

#### Constants Section
TEST_QUESTION = (4, 6)
QUESTION_TEMPLATE = "Combien font %s x %s ? "
LOWER = 1
UPPER = 12
MAX_QUESTIONS = 3 # for testing, you can increase it later
TIMES_TABLE_ENTRY = "%s x %s = %s"    #E1
TIMES_TABLE_ENTRY = "%2i x %2i = %3i " #E2, E3
INSTRUCTIONS = """Welcome to Math Trainer
This application will train you on your times tables.
It can either print one or more of the tables for you
so that you can revise (training) or it can test
your times tables.
""" #G
CONFIRM_QUIT_MESSAGE = 'Vraiment quitter (O/n)? ' #H
SCORE_TEMPLATE = "Ton score est de %s (%i%%) en %.1f secondes\n"

#### Function Section
def make_question_list(lower=LOWER, upper=UPPER, random_order=True):
    """ prepare a list of questions in the form (x,y)
    where x and y are in the range from LOWER to UPPER inclusive
    If random_order is true, rearrange the questions in a random order
    """
    spam = [(x+1, y+1) for x in range(lower - 1,upper)
                       for y in range(lower - 1,upper)]
    if random_order:
        random.shuffle(spam)
    return spam

def do_testing():
    """ conduct a round of testing """
    question_list = make_question_list()
    score = 0
    start_time = time.time() #I
    for i, question in enumerate(question_list):
        if i >= MAX_QUESTIONS:
            break
        prompt = QUESTION_TEMPLATE%question
        correct_answer = question[0]*question[1]
        # indexes start from 0
        answer = raw_input(prompt)

        if int(answer) == correct_answer:
            print("Correct !")
            score = score+1
        else:
            print("Incorrect, il fallait dire %s."%(correct_answer))

#H    print("Nombre de calculs corrects = %s"%score)
    end_time = time.time()
    time_taken = end_time-start_time
    percent_correct = int(score/float(MAX_QUESTIONS)*100)
    print(SCORE_TEMPLATE%(score, percent_correct, time_taken))

def display_times_tables(upper=UPPER): #F
    # Display the times tables up to UPPER
    tables_per_line = 6
    tables_to_print = range(1, upper+1)
    # get a batch of 5 or 6 to print
    batch = tables_to_print[:tables_per_line]
    # remove them from the list
    tables_to_print = tables_to_print[tables_per_line:]
    while batch != []: # stop when theres no more to print
        for x in range(1, upper+1):
            # this goes from 1 to 12 and is the rows
            accumulator = []
            for y in batch:
                # this covers only the tables in the batch
                # it builds the columns
                accumulator.append(TIMES_TABLE_ENTRY%(y, x, x*y))
            print("".join(accumulator)) # print one row
        print("\n") # vertical separation between blocks of tables.
        # now get another batch and repeat.
        batch = tables_to_print[:tables_per_line]
        tables_to_print = tables_to_print[tables_per_line:]

# def do_quit():  #G
#     print("In quit")

def do_quit():    #H
    """ quit the application"""
    if confirm_quit():
        sys.exit()
    print("In quit (not quitting, returning)")

def confirm_quit():
    """Ask user to confirm that they want to quit
    default to yes
    Return True (yes, quit) or False (no, dont quit) """
    spam = raw_input(CONFIRM_QUIT_MESSAGE)
    if spam == 'n':
        return False
    else:
        return True

#### Testing Section
# question = TEST_QUESTION
# prompt = QUESTION_TEMPLATE%question
# correct_answer = question[0]*question[1] # indexes start from 0
# answer = raw_input(prompt)
# if int(answer)== correct_answer:
#     print("Correct!")
# else:
#     print("Incorrect")
#B question_list = make_question_list()
#B print(question_list)

#C for lower,upper in [(2, 5), (4, 6), (7, 11)]:
#C     question_list = make_question_list(lower, upper)
#C     print(question_list)

#D do_testing()
#F display_times_tables()

#### Main Section
if __name__ == "__main__":
    while True:
        print(INSTRUCTIONS)
        raw_input_prompt = "Press: 1 for training,"+\
                        " 2 for testing, 3 to quit.\n"
        selection = raw_input(raw_input_prompt)
        selection = selection.strip()
        while selection not in ["1", "2", "3"]:
            selection = raw_input("Please type either 1, 2 or 3: ")
            selection = selection.strip()
        if selection == "1":
            display_times_tables()
        elif selection == "2":
            do_testing()
        else: # has to be 1, 2 or 3 so must be 3 (quit)
            do_quit()
